library(stockassessment)
load("run/model.RData")
source("src/forecasthelpers.R")

## Last year in data
yr <- as.numeric(tail(rownames(stockassessment::fbartable(fit)), 1))

## Forecast settings ----
Ry <- 2009:max(fit$data$years) ## Recruitment since 2009
Ay <- max(fit$data$years) + (-2:0)  ## Selectivity / biological vars years / landing fraction - last 3 years
forecastArgs <- list(ave.years=Ay, rec.years=Ry, splitLD=TRUE, nosim=1001)

## Last TAC advice ----
## TAC <- 2390000 # for 2018-2019
## TAC <- 1651000 # for 2020
TAC <- 1733       # for 2021

# Reference points ----
#Fmsy <- 0.154
#Flim <- 0.30
#Fpa <-  0.20
#Bpa <-  4745000
#Blim <- 3069000
#Btrigger <- 4745000
#Fmsyupper <- 0.21
#Fmsylower <- 0.108
#F05_AR <- 0.28 ## With Btrigger rule
#F05 <- 0.2 ## Without Btrigger rule
#Fpa <- min(F05, Fpa)
#Fmsyupper <- min(Fmsyupper, F05_AR)
#Fmsylower <- min(Fmsylower, F05_AR)

Fmsy <- 0.147
Flim <- 0.32
Fpa <-  0.28
Bpa <-  4381
Blim <- 3077
Btrigger <- 4381
Fmsyupper <- 0.20
Fmsylower <- 0.105
F05_AR <- 0.28 ## With Btrigger rule
F05 <- 0.22 ## Without Btrigger rule




## Fs for the scenarios ----
## Get Fsq from the assessment summary table
s <- summary(fit)
snm <- colnames(s)
Fsq <- s[nrow(s), startsWith(snm, "Fbar")]

## Find fishing mortality that leads to Blim, Bpa, Btrigger
tol <- 0.0001
F_Blim <- optimise(op, c(0, 1), target = Blim, tol = tol)$minimum
F_Bpa <- optimise(op, c(0, 1), target = Bpa, tol = tol)$minimum
F_Btr <- optimise(op, c(0, 1), target = Btrigger, tol = tol)$minimum

## Scenario definitions ----
# values in the vectors are for 2021:2024
scen <- list(
  "MSY approach: Fmsy" = list(fval = c(Fsq,Fsq,Fmsy,Fmsy)),
  "Precautionary approach: Fpa" = list( fval = c(Fsq,Fsq,Fpa,Fpa)),
  "FMSY upper" = list(fval = c(Fsq,Fsq,Fmsyupper,Fmsyupper)),
  "FMSY lower" = list(fval = c(Fsq,Fsq,Fmsylower,Fmsylower)),
  "F = 0" = list(fval = c(Fsq,Fsq,0.000001, 0.000001)),
  "Fpa" = list( fval = c(Fsq,Fsq,Fpa,Fpa)),
  "F = Flim" = list(fval = c(Fsq,Fsq,Flim,Flim)),
  "Fsq"  = list(fval = c(Fsq,Fsq,Fsq,Fsq)),
  "SSB (XXXX) = Blim" = list(fval = c(Fsq,Fsq,F_Blim,F_Blim)),
  "SSB (XXXX) = Bpa" = list(fval = c(Fsq,Fsq,F_Bpa,F_Bpa)),
  "SSB (XXXX) = MSY Btrigger" = list(fval = c(Fsq,Fsq,F_Btr,F_Btr)),
  "Rollover advice" = list(fval = c(Fsq,Fsq,NA, NA), catchval = c(NA,NA,TAC,TAC))
  )
  
names(scen) <- sub("XXXX", yr + 3, names(scen))

FC <- setNames(mapply(run_one_scenario, scen, names(scen), 
                      MoreArgs = forecastArgs,  SIMPLIFY = FALSE),
               names(scen))
## Make extra scenarios for a range of Fs
scen_extra <- make_scenarios(s = seq(0.01, 0.5, 0.01))
FCextra <- setNames(mapply(run_one_scenario, scen_extra, names(scen_extra), 
                           MoreArgs = forecastArgs, SIMPLIFY = FALSE),
               names(scen_extra))

save(FC, file="run/forecast.RData")
save(FCextra, file = "run/forecast_extra.RData")

## Advice sheet tables -----
load("run/forecast.RData")
load("run/forecast_extra.RData")
ft <- do.call(rbind.data.frame,lapply(FC, getForecastTable, catchyr = yr + 2, ssbyr = yr + 3, advice = TAC, splitLD=TRUE))
ftextra <- do.call(rbind.data.frame,lapply(FCextra, getForecastTable, catchyr = yr + 2, ssbyr = yr + 3, advice = TAC, splitLD=TRUE))

write.table(ftextra, file = "res/wit.27.3a47d_extraFscen.csv", sep = ",", row.names = FALSE)
at <- makeAssumtpionTable(intermyr = ay,2)

save(ft, file = paste0("run/wit.27.3a47d_forecast_table_", yr + 1, ".RData"))
save(at, file = paste0("run/wit.27.3a47d_forecast_assumptions_table_", yr + 1, ".RData"))

